<?php
/**
 * @file
 * Login include for the Watchdog Notifier module.
 *
 * This file has the breakout of the main function that does the watchdog
 * notifications.
 */

/**
 * Submit handler for the 'scan now' button on the config page.
 */
function watchdog_notifier_run_now_submit() {
  watchdog_notifier_do_that_thing();
  drupal_set_message(t('Watchdog log scan initiated.  Watchlist recipients should be notified of any new messages momentarily.'));
}


/**
 * The 'business logic' for watching the watchdog log.
 */
function watchdog_notifier_do_that_thing() {
  $total_messages_sent = 0;

  $watchlist;
  $query = db_select('watchdog_notifier','wn')->fields('wn');
  $result = $query->execute();
  foreach ($result as $res) {
    $watchlist[] = array(
      'wn_id'    => $res->wn_id,
      'email'    => $res->email,
      'type'     => $res->type,
      'severity' => $res->severity,
    );
  }

  $watchdog_notifier_last_run = variable_get('watchdog_notifier_last_run');

  $site_wide_email_address = variable_get('site_mail', '');
  $current_server          = $_SERVER['SERVER_NAME'];

  // Start the clock when we're ready to query for log messages.
  $start_time = time();

  foreach ($watchlist as $wl_entry) {
    $query = db_select('watchdog','w')
      ->fields('w')
      ->condition('type', $wl_entry['type'], '=')
      ->condition('severity', $wl_entry['severity'], '=')
      ->condition('timestamp', $watchdog_notifier_last_run, '>')
      ->orderBy('timestamp', 'ASC');
    $result = $query->execute();
    foreach ($result as $res) {
      $email_params = array(
        'subject' => '[Watchdog Notifier] ~' . $current_server . '~ type: ' . $res->type . ', severity: ' . watchdog_notifier_get_severity($res->severity),
        'body' => array("Watchdog log message contents:\n\n" . print_r($res, TRUE) . "\n"),
      );
      $output = drupal_mail(
        'watchdog_notifier',
        'information',
        $wl_entry['email'],
        language_default(),
        $email_params,
        $site_wide_email_address
      );
      $total_messages_sent += 1;
    }
  }

  $time_delta = time() - $start_time;
  variable_set('watchdog_notifier_last_run', $start_time + $time_delta);

  watchdog(
    'watchdog_notifier',
    'Finished scanning the watchdog log and emailing watchlist recipients. @num message(s) emailed. Runtime was @time seconds',
    array(
      '@num'  => print_r($total_messages_sent, TRUE),
      '@time' => print_r($time_delta, TRUE),
    ),
    WATCHDOG_INFO
  );
}


/**
 * Implements hook_mail().
 */
function watchdog_notifier_mail($key, &$message, $params) {
  switch ($key) {
    case 'information':
      $message['subject'] = $params['subject'];
      $message['body']    = $params['body'];
    break;
    }
}

/**
 * Quick function to return the string equivalent of each of numerical severity
 * level.
 */
function watchdog_notifier_get_severity($notification_level = null) {
  $severity_array = array(
    '0' => 'Emergency',
    '1' => 'Alert',
    '2' => 'Critical',
    '3' => 'Error',
    '4' => 'Warning',
    '5' => 'Notice',
    '6' => 'Informational',
    '7' => 'Debug',
  );

  $severity_string = '';
  if ($notification_level == null) {
    $severity_string = 'No severity level has been provided.';
  } else {
    $severity_string = $severity_array[$notification_level];
  }

  return $severity_string;
}
